# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.event_trigger_type import EventTriggerType
from ..core.request_options import RequestOptions
from ..commons.types.event_trigger_response import EventTriggerResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.error_message import ErrorMessage
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.server_error import ServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TriggersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_or_update(
        self,
        *,
        trigger_id: EntityIdBase,
        description: str,
        type: EventTriggerType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventTriggerResponse:
        """
        Update an event trigger or create it if it doesn't exist.

        Parameters
        ----------
        trigger_id : EntityIdBase
            ID that uniquely identifies this event trigger

        description : str
            The description of what the event trigger does, shown in the Maven Dashboard

        type : EventTriggerType
            The type of event trigger this app wishes to handle.

            Conversation triggers fire when a conversation is created, after each additional message, and upon deletion events.
            There is a small delay before trigger execution to allow time for conversation analysis to complete.

            Feedback can not be modified, so the feedback trigger fires immediately after feedback is created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventTriggerResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.triggers.create_or_update(
            trigger_id=EntityIdBase(
                reference_id="store-in-snowflake",
            ),
            description="Stores conversation data in Snowflake",
            type="CONVERSATION_CREATED",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/triggers",
            method="PUT",
            json={
                "triggerId": convert_and_respect_annotation_metadata(
                    object_=trigger_id, annotation=EntityIdBase, direction="write"
                ),
                "description": description,
                "type": type,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EventTriggerResponse,
                    parse_obj_as(
                        type_=EventTriggerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, trigger_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EventTriggerResponse:
        """
        Get an event trigger by its supplied ID

        Parameters
        ----------
        trigger_reference_id : str
            The reference ID of the event trigger to get. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventTriggerResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.triggers.get(
            trigger_reference_id="store-in-snowflake",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/triggers/{jsonable_encoder(trigger_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EventTriggerResponse,
                    parse_obj_as(
                        type_=EventTriggerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, trigger_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete an event trigger

        Parameters
        ----------
        trigger_reference_id : str
            The reference ID of the event trigger to delete. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.triggers.delete(
            trigger_reference_id="store-in-snowflake",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/triggers/{jsonable_encoder(trigger_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTriggersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_or_update(
        self,
        *,
        trigger_id: EntityIdBase,
        description: str,
        type: EventTriggerType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventTriggerResponse:
        """
        Update an event trigger or create it if it doesn't exist.

        Parameters
        ----------
        trigger_id : EntityIdBase
            ID that uniquely identifies this event trigger

        description : str
            The description of what the event trigger does, shown in the Maven Dashboard

        type : EventTriggerType
            The type of event trigger this app wishes to handle.

            Conversation triggers fire when a conversation is created, after each additional message, and upon deletion events.
            There is a small delay before trigger execution to allow time for conversation analysis to complete.

            Feedback can not be modified, so the feedback trigger fires immediately after feedback is created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventTriggerResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.triggers.create_or_update(
                trigger_id=EntityIdBase(
                    reference_id="store-in-snowflake",
                ),
                description="Stores conversation data in Snowflake",
                type="CONVERSATION_CREATED",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/triggers",
            method="PUT",
            json={
                "triggerId": convert_and_respect_annotation_metadata(
                    object_=trigger_id, annotation=EntityIdBase, direction="write"
                ),
                "description": description,
                "type": type,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EventTriggerResponse,
                    parse_obj_as(
                        type_=EventTriggerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, trigger_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EventTriggerResponse:
        """
        Get an event trigger by its supplied ID

        Parameters
        ----------
        trigger_reference_id : str
            The reference ID of the event trigger to get. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventTriggerResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.triggers.get(
                trigger_reference_id="store-in-snowflake",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/triggers/{jsonable_encoder(trigger_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EventTriggerResponse,
                    parse_obj_as(
                        type_=EventTriggerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, trigger_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete an event trigger

        Parameters
        ----------
        trigger_reference_id : str
            The reference ID of the event trigger to delete. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.triggers.delete(
                trigger_reference_id="store-in-snowflake",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/triggers/{jsonable_encoder(trigger_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .series import Series
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BarChartResponse(UniversalBaseModel):
    series: typing.List[Series] = pydantic.Field()
    """
    Each LabeledPoint in the series represents a value for the bar, where the name serves as the bar's label.
    If vertical grouping is defined, multiple series will be created, each representing a group.
    In such cases, the bars can be stacked to reflect the grouped data distribution.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .app_user_identifier import AppUserIdentifier
import pydantic
from .user_data import UserData
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AppUser(UniversalBaseModel):
    identifiers: typing.List[AppUserIdentifier] = pydantic.Field()
    """
    Used to determine whether two users from different apps are the same
    """

    data: typing.Dict[str, UserData]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AskRequest,
    AskStreamActionEvent,
    AskStreamChartEvent,
    AskStreamEndEvent,
    AskStreamMetadataEvent,
    AskStreamStartEvent,
    AskStreamTextEvent,
    Attachment,
    CategorizationResponse,
    ConversationField,
    ConversationFilter,
    ConversationMessageRequest,
    ConversationRequest,
    FeedbackField,
    FeedbackFilter,
    FeedbackRequest,
    GenerateMavenSuggestionsRequest,
    NumericConversationField,
    Quality,
    QualityReason,
    StreamResponse,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_End,
    StreamResponse_Metadata,
    StreamResponse_Start,
    StreamResponse_Text,
    SubmitActionFormRequest,
)

__all__ = [
    "AskRequest",
    "AskStreamActionEvent",
    "AskStreamChartEvent",
    "AskStreamEndEvent",
    "AskStreamMetadataEvent",
    "AskStreamStartEvent",
    "AskStreamTextEvent",
    "Attachment",
    "CategorizationResponse",
    "ConversationField",
    "ConversationFilter",
    "ConversationMessageRequest",
    "ConversationRequest",
    "FeedbackField",
    "FeedbackFilter",
    "FeedbackRequest",
    "GenerateMavenSuggestionsRequest",
    "NumericConversationField",
    "Quality",
    "QualityReason",
    "StreamResponse",
    "StreamResponse_Action",
    "StreamResponse_Chart",
    "StreamResponse_End",
    "StreamResponse_Metadata",
    "StreamResponse_Start",
    "StreamResponse_Text",
    "SubmitActionFormRequest",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from .knowledge_base_properties import KnowledgeBaseProperties
from ...commons.types.precondition_group import PreconditionGroup
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
from ...core.pydantic_utilities import update_forward_refs


class KnowledgeBaseRequest(KnowledgeBaseProperties):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase
    from mavenagi.knowledge import KnowledgeBaseRequest

    KnowledgeBaseRequest(
        knowledge_base_id=EntityIdBase(
            reference_id="help-center",
        ),
        name="Help center",
        type="API",
    )
    """

    knowledge_base_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="knowledgeBaseId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge base
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(PreconditionGroup, KnowledgeBaseRequest=KnowledgeBaseRequest)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
from .precondition_group import PreconditionGroup
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
import typing
from .precondition import Precondition
from .action_parameter import ActionParameter
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.pydantic_utilities import update_forward_refs


class ActionBase(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the action. This is displayed to the end user as part of forms when user interaction is required. It is also used to help Maven decide if the action is relevant to a conversation.
    """

    description: str = pydantic.Field()
    """
    The description of the action. Must be less than 1024 characters. This helps Maven decide if the action is relevant to a conversation and is not displayed directly to the end user. Descriptions are used by the LLM.
    """

    user_interaction_required: typing_extensions.Annotated[bool, FieldMetadata(alias="userInteractionRequired")] = (
        pydantic.Field()
    )
    """
    Whether the action requires user interaction to execute. If false, and all of the required action parameters are known, the LLM may call the action automatically. If true, an conversations ask call will return a BotActionFormResponse which must be submitted by an API caller. API callers must display a button with the buttonName label to confirm the user's intent.
    """

    button_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="buttonName")] = pydantic.Field(
        default=None
    )
    """
    When user interaction is required, the name of the button that is shown to the end user to confirm execution of the action
    """

    precondition: typing.Optional[Precondition] = pydantic.Field(default=None)
    """
    The preconditions that must be met for an action to be relevant to a conversation. Can be used to restrict actions to certain types of users.
    """

    user_form_parameters: typing_extensions.Annotated[
        typing.List[ActionParameter], FieldMetadata(alias="userFormParameters")
    ] = pydantic.Field()
    """
    The parameters that the action uses as input. An action will only be executed when all of the required parameters are provided. During execution, actions all have access to the full Conversation and User objects. Parameter values may be inferred from the user's conversation by the LLM.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(PreconditionGroup, ActionBase=ActionBase)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class CellData_Double(UniversalBaseModel):
    value: float
    type: typing.Literal["double"] = "double"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CellData_Long(UniversalBaseModel):
    value: int
    type: typing.Literal["long"] = "long"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CellData_Millisecond(UniversalBaseModel):
    value: float
    type: typing.Literal["millisecond"] = "millisecond"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CellData_String(UniversalBaseModel):
    value: str
    type: typing.Literal["string"] = "string"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


CellData = typing.Union[CellData_Double, CellData_Long, CellData_Millisecond, CellData_String]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .time_data_point import TimeDataPoint
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TimeSeries(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Name of the series, derived from the grouping field or percentile metric.
    """

    data: typing.List[TimeDataPoint] = pydantic.Field()
    """
    List of time-based data points for the series.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from .response_config import ResponseConfig
from ...core.serialization import FieldMetadata
import pydantic
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationBase(UniversalBaseModel):
    response_config: typing_extensions.Annotated[
        typing.Optional[ResponseConfig], FieldMetadata(alias="responseConfig")
    ] = pydantic.Field(default=None)
    """
    Optional configurations for responses to this conversation
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subject of the conversation
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The url of the conversation
    """

    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was created
    """

    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was last updated
    """

    tags: typing.Optional[typing.Set[str]] = pydantic.Field(default=None)
    """
    The tags of the conversation. Used for filtering in Agent Designer.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    The metadata of the conversation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .labeled_point import LabeledPoint
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Series(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the series, derived from the grouping field.
    If the metric is a percentile, the name represents the percentile value.
    """

    data: typing.List[LabeledPoint] = pydantic.Field()
    """
    List of labeled data points for the series.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
import typing
from .attachment import Attachment
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AskRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase
    from mavenagi.conversation import AskRequest, Attachment

    AskRequest(
        conversation_message_id=EntityIdBase(
            reference_id="message-0",
        ),
        user_id=EntityIdBase(
            reference_id="user-0",
        ),
        text="How do I reset my password?",
        attachments=[
            Attachment(
                type="image/png",
                content="iVBORw0KGgo...",
            )
        ],
        transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
    )
    """

    conversation_message_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.
    """

    user_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    Externally supplied ID to uniquely identify the user that created this message
    """

    text: str = pydantic.Field()
    """
    The text of the message
    """

    attachments: typing.Optional[typing.List[Attachment]] = pydantic.Field(default=None)
    """
    The attachments to the message.
    """

    transient_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="transientData")
    ] = pydantic.Field(default=None)
    """
    Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import typing_extensions
import datetime as dt
from ...core.serialization import FieldMetadata
from ...commons.types.entity_id_filter import EntityIdFilter
from ...commons.types.feedback_type import FeedbackType
from .quality import Quality
from .quality_reason import QualityReason
from ...commons.types.response_length import ResponseLength
from ...commons.types.sentiment import Sentiment
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ConversationFilter(UniversalBaseModel):
    search: typing.Optional[str] = None
    created_after: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAfter")] = None
    created_before: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdBefore")] = (
        None
    )
    apps: typing.Optional[typing.List[str]] = None
    categories: typing.Optional[typing.List[str]] = None
    actions: typing.Optional[typing.List[EntityIdFilter]] = None
    incomplete_actions: typing_extensions.Annotated[
        typing.Optional[typing.List[EntityIdFilter]], FieldMetadata(alias="incompleteActions")
    ] = None
    feedback: typing.Optional[typing.List[FeedbackType]] = None
    human_agents: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="humanAgents")] = (
        None
    )
    languages: typing.Optional[typing.List[str]] = None
    quality: typing.Optional[typing.List[Quality]] = None
    quality_reason: typing_extensions.Annotated[
        typing.Optional[typing.List[QualityReason]], FieldMetadata(alias="qualityReason")
    ] = None
    response_length: typing_extensions.Annotated[
        typing.Optional[typing.List[ResponseLength]], FieldMetadata(alias="responseLength")
    ] = None
    sentiment: typing.Optional[typing.List[Sentiment]] = None
    tags: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import datetime as dt
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata


class FieldValue_DateTime(UniversalBaseModel):
    value: dt.datetime
    type: typing.Literal["dateTime"] = "dateTime"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class FieldValue_String(UniversalBaseModel):
    value: str
    type: typing.Literal["string"] = "string"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class FieldValue_Double(UniversalBaseModel):
    value: float
    type: typing.Literal["double"] = "double"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class FieldValue_Long(UniversalBaseModel):
    value: int
    type: typing.Literal["long"] = "long"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class FieldValue_Range(UniversalBaseModel):
    type: typing.Literal["range"] = "range"
    from_: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="from")] = None
    to: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FieldValue_Boolean(UniversalBaseModel):
    value: bool
    type: typing.Literal["boolean"] = "boolean"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class FieldValue_EntityId(UniversalBaseModel):
    type: typing.Literal["entityId"] = "entityId"
    reference_id: typing_extensions.Annotated[str, FieldMetadata(alias="referenceId")]
    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


FieldValue = typing.Union[
    FieldValue_DateTime,
    FieldValue_String,
    FieldValue_Double,
    FieldValue_Long,
    FieldValue_Range,
    FieldValue_Boolean,
    FieldValue_EntityId,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .capability import Capability
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
from .response_length import ResponseLength
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ResponseConfig(UniversalBaseModel):
    capabilities: typing.List[Capability] = pydantic.Field()
    """
    List of capabilities supported by the caller. Defaults to [MARKDOWN, FORMS, IMAGES].
    
    MARKDOWN: Whether the response should include markdown formatting. If not provided, the response will be plain text. Not respected while streaming.
    
    FORMS: Whether the response should include forms. If provided, the caller needs to render action forms when returned from the ask API and allow submission of the forms with the submitActionForm API. If not provided, then actions which require user interaction will not be considered by the LLM. Removing this capability is recommended for surfaces which can not display UI (e.g. SMS, voice).
    
    IMAGES: Whether the response should include images. Not yet supported.
    
    CHARTS_HIGHCHARTS_TS: Whether the response should include a Highcharts typescript chart if applicable.
    """

    is_copilot: typing_extensions.Annotated[bool, FieldMetadata(alias="isCopilot")] = pydantic.Field()
    """
    Whether the response is for an human agent (true) or an end user (false). Defaults to false.
    """

    response_length: typing_extensions.Annotated[ResponseLength, FieldMetadata(alias="responseLength")] = (
        pydantic.Field()
    )
    """
    The desired response length. Defaults to ResponseLength.MEDIUM.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .precondition_base import PreconditionBase
import typing_extensions
import typing
from ...core.serialization import FieldMetadata
from .response_length import ResponseLength
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ResponseConfigPrecondition(PreconditionBase):
    use_markdown: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useMarkdown")] = None
    use_forms: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useForms")] = None
    use_images: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useImages")] = None
    is_copilot: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="isCopilot")] = None
    response_length: typing_extensions.Annotated[
        typing.Optional[ResponseLength], FieldMetadata(alias="responseLength")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

QualityReason = typing.Union[
    typing.Literal[
        "MISSING_KNOWLEDGE",
        "MISSING_USER_INFORMATION",
        "MISSING_ACTION",
        "NEEDS_USER_CLARIFICATION",
        "UNSUPPORTED_FORMAT",
        "GREAT_ANSWER",
        "INTERRUPTED",
        "UNSUPPORTED_USER_BEHAVIOR",
        "UNKNOWN",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .row_base import RowBase
import typing
from ...conversation.types.conversation_field import ConversationField
from .field_value import FieldValue
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationRow(RowBase):
    identifier: typing.Dict[ConversationField, FieldValue] = pydantic.Field()
    """
    A unique identifier for each row, consisting of field names mapped to their respective values.
    This includes time groupings and any specified field groupings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import typing_extensions
from .conversation_group_by import ConversationGroupBy
from ...core.serialization import FieldMetadata
from .conversation_metric import ConversationMetric
from ...conversation.types.conversation_filter import ConversationFilter
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .time_interval import TimeInterval


class ConversationChartRequest_PieChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        ConversationChartRequest_PieChart,
        ConversationGroupBy,
        ConversationMetric_Count,
    )
    from mavenagi.conversation import ConversationFilter

    ConversationChartRequest_PieChart(
        conversation_filter=ConversationFilter(
            languages=["en", "es"],
        ),
        group_by=ConversationGroupBy(
            field="Category",
        ),
        metric=ConversationMetric_Count(),
    )
    """

    type: typing.Literal["pieChart"] = "pieChart"
    group_by: typing_extensions.Annotated[ConversationGroupBy, FieldMetadata(alias="groupBy")]
    metric: ConversationMetric
    conversation_filter: typing_extensions.Annotated[
        typing.Optional[ConversationFilter], FieldMetadata(alias="conversationFilter")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationChartRequest_DateHistogram(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        ConversationChartRequest_PieChart,
        ConversationGroupBy,
        ConversationMetric_Count,
    )
    from mavenagi.conversation import ConversationFilter

    ConversationChartRequest_PieChart(
        conversation_filter=ConversationFilter(
            languages=["en", "es"],
        ),
        group_by=ConversationGroupBy(
            field="Category",
        ),
        metric=ConversationMetric_Count(),
    )
    """

    type: typing.Literal["dateHistogram"] = "dateHistogram"
    time_interval: typing_extensions.Annotated[TimeInterval, FieldMetadata(alias="timeInterval")]
    group_by: typing_extensions.Annotated[typing.Optional[ConversationGroupBy], FieldMetadata(alias="groupBy")] = None
    metric: ConversationMetric
    conversation_filter: typing_extensions.Annotated[
        typing.Optional[ConversationFilter], FieldMetadata(alias="conversationFilter")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationChartRequest_BarChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        ConversationChartRequest_PieChart,
        ConversationGroupBy,
        ConversationMetric_Count,
    )
    from mavenagi.conversation import ConversationFilter

    ConversationChartRequest_PieChart(
        conversation_filter=ConversationFilter(
            languages=["en", "es"],
        ),
        group_by=ConversationGroupBy(
            field="Category",
        ),
        metric=ConversationMetric_Count(),
    )
    """

    type: typing.Literal["barChart"] = "barChart"
    bar_definition: typing_extensions.Annotated[ConversationGroupBy, FieldMetadata(alias="barDefinition")]
    metric: ConversationMetric
    vertical_grouping: typing_extensions.Annotated[
        typing.Optional[ConversationGroupBy], FieldMetadata(alias="verticalGrouping")
    ] = None
    conversation_filter: typing_extensions.Annotated[
        typing.Optional[ConversationFilter], FieldMetadata(alias="conversationFilter")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mavenagi.analytics import (
    ConversationChartRequest_PieChart,
    ConversationGroupBy,
    ConversationMetric_Count,
)
from mavenagi.conversation import ConversationFilter

ConversationChartRequest_PieChart(
    conversation_filter=ConversationFilter(
        languages=["en", "es"],
    ),
    group_by=ConversationGroupBy(
        field="Category",
    ),
    metric=ConversationMetric_Count(),
)
"""
ConversationChartRequest = typing.Union[
    ConversationChartRequest_PieChart, ConversationChartRequest_DateHistogram, ConversationChartRequest_BarChart
]



# This file was auto-generated by Fern from our API Definition.

from .entity_id_base import EntityIdBase
from .entity_type import EntityType
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EntityIdWithoutAgent(EntityIdBase):
    """
    The organizationId and agentId are inferred from the context.
    """

    type: EntityType = pydantic.Field()
    """
    The object type
    """

    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")] = pydantic.Field()
    """
    The ID of the application that created this object
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .group_by_base import GroupByBase
from ...conversation.types.feedback_field import FeedbackField
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class FeedbackGroupBy(GroupByBase):
    field: FeedbackField = pydantic.Field()
    """
    Field used for data grouping.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .bad_request_error import BadRequestError
from .not_found_error import NotFoundError
from .server_error import ServerError

__all__ = ["BadRequestError", "NotFoundError", "ServerError"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from ...conversation.types.feedback_field import FeedbackField
from ...core.serialization import FieldMetadata


class FeedbackMetric_Count(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    """

    type: typing.Literal["count"] = "count"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FeedbackMetric_DistinctCount(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    """

    type: typing.Literal["distinctCount"] = "distinctCount"
    target_field: typing_extensions.Annotated[FeedbackField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


FeedbackMetric = typing.Union[FeedbackMetric_Count, FeedbackMetric_DistinctCount]
